#ifndef GRID_H
#define GRID_H

#include <stdint.h>
#include <stdio.h>
#include <set>
#include "CellState.h"
#include "State.h"
#include "Pos.h"

class Trampoline
{
public:
    Trampoline()
        : m_pos(), m_target(0) {}
    ~Trampoline() {}

    Pos m_pos;
    uint32_t m_target;
};

class Target
{
public:
    Target()
        : m_pos() {}
    ~Target() {}

    Pos m_pos;
};

class GridInfo
{
public:
    GridInfo();
    ~GridInfo();

    uint32_t m_time;
    int32_t m_score;
    State m_state;
    uint32_t m_water;
    uint32_t m_underwater;
    uint32_t m_lambdaCollected;
    uint32_t m_razors;
    Pos m_robotPos;
    std::set<Pos> m_fallingRocks;
    std::set<Pos> m_growingBeards;
};

class Grid
{
public:
    Grid();
    ~Grid();

    void resize(uint32_t width, uint32_t height);

    const CellState& cell(uint32_t x, uint32_t y) const {
        return m_data[y * (m_width+2) + x];
    }
    CellState& cell(uint32_t x, uint32_t y) {
        return m_data[y * (m_width+2) + x];
    }

    const CellState& cell(Pos pos) const {
        return m_data[pos.m_y * (m_width+2) + pos.m_x];
    }
    CellState& cell(Pos pos) {
        return m_data[pos.m_y * (m_width+2) + pos.m_x];
    }

    GridInfo* m_pInfo;
    uint32_t m_width;
    uint32_t m_height;
    CellState* m_data;
    Pos m_liftPos;
    uint32_t m_flooding;
    uint32_t m_waterproof;
    uint32_t m_lambdaTotal;
    uint32_t m_growth;
    Trampoline m_trampolines[9];
    Target m_targets[9];
};

bool readGrid(FILE* f, Grid* pGrid);

void writeGrid(FILE* f, const Grid& grid, bool ansi);

#endif
